package pigbarf;

import java.io.*;
import java.util.*;

public abstract class AbstractParser {
   protected class Result<T> {
      private final boolean failed;
      private final T result;
      private final String errorMessage;
      private final Result<?> inner;
      private final Position position;
      public Result(Result<?> _inner, String _errorMessage, Position _position) {
         this.position = _position;
         this.failed = true;
         this.result = null;
         this.inner = _inner;
         this.errorMessage = _errorMessage;
      }
      public Result(T _result) {
         this.failed = false;
         this.result = _result;
         this.position = null;
         this.inner = null;
         this.errorMessage = null;
      }
      public Result<?> getInnerResult() {
         if (!this.failed)
            throw new UnsupportedOperationException();
         else
            return this.inner;
      }
      public String getErrorMessage() {
        if (!this.failed)
            throw new UnsupportedOperationException();
         else
            return this.errorMessage;
      }
      public Position getPosition() {
         if (!this.failed)
            throw new UnsupportedOperationException();
         else
            return this.position;
      }
      public boolean failed() {return this.failed;}
      public T result() {
         if (this.failed)
            throw new UnsupportedOperationException();
         else
            return this.result;
      }
   }

   protected <T> Result<T> _yy_success(T result) {
      return new Result<T>(result);
   }
   protected <T> Result<T> _yy_fail(String _msg) {
      return new Result<T>(null, _msg, _yy_getPosition());
   }
   protected <T> Result<T> _yy_fail(Result<?> inner) {
      return new Result<T>(inner, null, _yy_getPosition());
   }
   protected <T> Result<T> _yy_fail(String msg, Result<?> inner) {
      return new Result<T>(inner, msg, _yy_getPosition());
   }

   protected String _yy_getResultMessage(Result<?> result) {
      Result<?> inner = result.getInnerResult();
      String msg = result.getErrorMessage();
      if (inner != null) {
         // has an inner, may have a string
         String innerMsg = _yy_getResultMessage(inner);
         if (msg != null) {
            return "\n" + result.getPosition() + " " + msg + ": " + innerMsg;
         } else {
            return innerMsg;
         }
      } else {
         // has a string
         return "\n" + result.getPosition() + " " + msg;
      } 
   }

   protected static class Position {
      public final int bitIndex, byteIndex;
      public Position(int _bitIndex, int _byteIndex) {
         this.bitIndex = _bitIndex;
         this.byteIndex = _byteIndex;
      }
      public String toString() {
         if (this.bitIndex != 0) {
            return "[" + this.byteIndex + ":" + this.bitIndex + "]";
         } else {
            return "[" + this.byteIndex + "]";
         }
      }
   }

   protected interface MiniParser<T> {
      Result<T> get() throws IOException, ParsingException;
   }

   /////////////////////////////////////////

   private static final int _YY_BUFFER_SIZE = 65536;
   private final List<byte[]> _yy_data = 
      new ArrayList<byte[]>();
   private final InputStream _yy_inputStream;
   private int _yy_dataRead = 0;
   private int _yy_byteIndex = 0;
   private int _yy_bitIndex = 0; // always in [0,8)
   private boolean _yy_byteOrderBE;
   private boolean _yy_bitOrderBE;
   private boolean _yy_sawEOF = false;

   public AbstractParser(InputStream _inputStream, 
                         boolean _byteOrderBE, 
                         boolean _bitOrderBE) {
      this._yy_inputStream = _inputStream;
      this._yy_byteOrderBE = _byteOrderBE;
      this._yy_bitOrderBE = _bitOrderBE;
      this._yy_data.add(new byte[_YY_BUFFER_SIZE]);
   }
   public AbstractParser(byte[] _data,
                         boolean _byteOrderBE, 
                         boolean _bitOrderBE) {
      this._yy_inputStream = null;
      this._yy_byteOrderBE = _byteOrderBE;
      this._yy_bitOrderBE = _bitOrderBE;
      int index = 0;
      while (index < _data.length) {
         int tocopy = Math.min(_YY_BUFFER_SIZE, _data.length-index);
         byte[] buffer = new byte[_YY_BUFFER_SIZE];
         System.arraycopy(_data, index, buffer, 0, tocopy);
         this._yy_data.add(buffer);
         index += tocopy;
      }
      this._yy_dataRead = index;
      this._yy_sawEOF = true;
   }

   protected void _yy_setByteOrder(boolean isBigEndian) {
      this._yy_byteOrderBE = isBigEndian;
   }
   protected void _yy_setBitOrder(boolean isBigEndian) {
      this._yy_bitOrderBE = isBigEndian;
   }

   protected Position _yy_getPosition() {
      return new Position(this._yy_bitIndex, this._yy_byteIndex);
   }
   protected void _yy_setPosition(Position pos) {
      this._yy_bitIndex = pos.bitIndex;
      this._yy_byteIndex = pos.byteIndex;
   }

   protected boolean _yy_align(int boundary) throws IOException, ParsingException {
      if (boundary <= 0)
         throw new ParsingException("Alignment value must be positive: " + boundary);
      if (this._yy_bitIndex != 0) {
         this._yy_byteIndex++;
         this._yy_bitIndex = 0;
      }
      int size = this._yy_byteIndex % boundary;
      byte[] scrap = new byte[(boundary-size)%boundary];
      return this._yy_getN(scrap);
   }
   

   /** 
    * Tries to read in a whole buffer's worth,
    * Returns false at EOF.
    */
   private boolean _yy_read() throws IOException, ParsingException {
      if (this._yy_sawEOF) return false;
      
      int start = this._yy_dataRead % _YY_BUFFER_SIZE;
      int whichbuffer = this._yy_dataRead / _YY_BUFFER_SIZE;
      byte[] buffer;
      if (whichbuffer == this._yy_data.size()) {
         // start new buffer
         buffer = new byte[_YY_BUFFER_SIZE];
         this._yy_data.add(buffer);
      } else {
         // finish last buffer
         buffer = this._yy_data.get(whichbuffer);
      }

      // fill buffer!
      int read = this._yy_inputStream.read(buffer, start, buffer.length-start);
      if (read == -1) {
         this._yy_sawEOF = true;
         return false;
      } else {
         this._yy_dataRead += read;
         return true;
      }
   }

   private boolean _yy_getN(byte[] bytes) throws IOException, ParsingException {
      if (this._yy_bitIndex != 0) {
         this._yy_byteIndex++;
         this._yy_bitIndex = 0;
      }
      while (this._yy_byteIndex + bytes.length > this._yy_dataRead &&
             this._yy_read()) {}
      if (this._yy_byteIndex + bytes.length > this._yy_dataRead)
         return false;
      for (int i = 0; i < bytes.length; i++) {
         int buffer = this._yy_byteIndex / _YY_BUFFER_SIZE;
         int index = this._yy_byteIndex % _YY_BUFFER_SIZE;
         bytes[i] = this._yy_data.get(buffer)[index];
         this._yy_byteIndex++;
      }
      return true;
   }

   ////////// Primitives ///////////////////////////
   
   protected class _yy_bytes implements MiniParser<byte[]> {
      private final int length;
      private final boolean hasValue;
      private final byte[] value;
      public _yy_bytes(int _length) throws IOException, ParsingException {
         this(_length, null);
      }
      public _yy_bytes(int _length, byte[] _value) throws IOException, ParsingException {
         if (_length < 0)
            throw new ParsingException("Negative number of bytes specified: " + _length);
         this.length = _length;
         this.hasValue = (_value != null);
         this.value = _value;
      }
      public Result<byte[]> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         byte[] result = new byte[this.length];
         if (!_yy_getN(result)) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'bytes' primitive");
         } else {
            if (hasValue) {
               if (result.length != value.length) {
                  _yy_setPosition(start);
                  return _yy_fail("Value for 'bytes' does not match");
               }
               for (int i = 0; i < result.length; i++) {
                  if (result[i] != value[i]) {
                     _yy_setPosition(start);
                     return _yy_fail("Value for 'bytes' does not match");
                  }
               }
            }
            return _yy_success(result);
         }
      }
   }

   private final byte[] _YY_BYTE_BUFFER = new byte[1];
   protected class _yy_byte implements MiniParser<Byte> {
      private final byte value;
      private final boolean hasValue;
      public _yy_byte(byte _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public _yy_byte() {
         this.hasValue = false;
         this.value = 0;
      }
      public Result<Byte> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         if (!_yy_getN(_YY_BYTE_BUFFER)) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'byte' primitive");
         } else {
            if (this.hasValue && _YY_BYTE_BUFFER[0] != this.value) {
               _yy_setPosition(start);
               return _yy_fail("Value for 'byte' does not match: " + _YY_BYTE_BUFFER[0] + " != " + this.value);
            }
            return _yy_success(_YY_BYTE_BUFFER[0]);
         }
      }
   }
      
   private final byte[] _YY_INT_BUFFER = new byte[4];
   protected class _yy_int implements MiniParser<Integer> {
      private final boolean hasValue;
      private final int value;
      public _yy_int() {
         this.hasValue = false;
         this.value = 0;
      }
      public _yy_int(int _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         if (!_yy_getN(_YY_INT_BUFFER)) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'int' primitive");
         }
         int result;
         if (_yy_byteOrderBE) {
            result = 
               ((_YY_INT_BUFFER[0]&0xFF)<<24) | ((_YY_INT_BUFFER[1]&0xFF)<<16) | 
               ((_YY_INT_BUFFER[2]&0xFF)<<8)  | (_YY_INT_BUFFER[3]&0xFF);
         } else {
            result = 
               ((_YY_INT_BUFFER[3]&0xFF)<<24) | ((_YY_INT_BUFFER[2]&0xFF)<<16) | 
               ((_YY_INT_BUFFER[1]&0xFF)<<8)  | (_YY_INT_BUFFER[0]&0xFF);
         }
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'int' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   private final byte[] _YY_SHORT_BUFFER = new byte[2]; 
   protected class _yy_short implements MiniParser<Short> {
      private final boolean hasValue;
      private final short value;
      public _yy_short() {
         this.hasValue = false;
         this.value = 0;
      }
      public _yy_short(short _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Short> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         if (!_yy_getN(_YY_SHORT_BUFFER)) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'short' primitive");
         }
         short result;
         if (_yy_byteOrderBE) {
            result = (short)( ((_YY_SHORT_BUFFER[0]&0xFF)<<8) | (_YY_SHORT_BUFFER[1]&0xFF) );
         } else {
            result = (short)( ((_YY_SHORT_BUFFER[1]&0xFF)<<8) | (_YY_SHORT_BUFFER[0]&0xFF) );
         }
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'short' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_long implements MiniParser<Long> {
      private final boolean hasValue;
      private final long value;
      private final _yy_int inner = new _yy_int();
      public _yy_long() {
         this.hasValue = false;
         this.value = 0L;
      }
      public _yy_long(long _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         Result<Integer> first = inner.get();
         Result<Integer> second = inner.get();
         if (first.failed() || second.failed()) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'long' primitive");
         }
         long result;
         if (_yy_byteOrderBE) {
            result = ((first.result()&0xFFFFFFFFL)<<32L) | (second.result()&0xFFFFFFFFL);
         } else {
            result = ((second.result()&0xFFFFFFFFL)<<32L) | (first.result()&0xFFFFFFFFL);
         }
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'long' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_float implements MiniParser<Float> {
      private final boolean hasValue;
      private final float value;
      private final _yy_int inner = new _yy_int();
      public _yy_float() {
         this.hasValue = false;
         this.value = 0.0f;
      }
      public _yy_float(float _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Float> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         Result<Integer> first = this.inner.get();
         if (first.failed()) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'float' primitive");
         }
         float result = Float.intBitsToFloat(first.result());
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'float' does not match: " + result  + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_double implements MiniParser<Double> {
      private final boolean hasValue;
      private final double value;
      private final _yy_long inner = new _yy_long();
      public _yy_double() {
         this.hasValue = false;
         this.value = 0.0;
      }
      public _yy_double(double _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Double> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         Result<Long> first = this.inner.get();
         if (first.failed()) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'double' primitive");
         }
         double result = Double.longBitsToDouble(first.result());
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'double' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_char implements MiniParser<Character> {
      private final boolean hasValue;
      private final char value;
      private final _yy_byte inner = new _yy_byte();
      public _yy_char() {
         this.hasValue = false;
         this.value = 0;
      }
      public _yy_char(char _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Character> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         Result<Byte> first = this.inner.get();
         if (first.failed()) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'char' primitive");
         }
         char result = (char)(first.result()&0xFF);
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'char' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_chars implements MiniParser<char[]> {
      private final int length;
      private final boolean hasValue;
      private final char[] value;
      public _yy_chars(int _length) throws IOException, ParsingException {
         this(_length, null);
      }
      public _yy_chars(int _length, char[] _value) throws IOException, ParsingException {
         if (_length < 0)
            throw new ParsingException("Negative number of chars specified: " + _length);
         this.length = _length;
         this.hasValue = (_value != null);
         this.value = _value;
      }
      public Result<char[]> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         byte[] bytes = new byte[this.length];
         if (!_yy_getN(bytes)) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'chars' primitive");
         }
         char[] result = new char[this.length];
         for (int i = 0; i < this.length; i++) {
            result[i] = (char)(bytes[i]&0xFF);
         }
         if (this.hasValue) {
            if (result.length != this.value.length) {
               _yy_setPosition(start);
               return _yy_fail("Value for 'chars' does not match: " + new String(result) + " != " + new String(this.value));
            }
            for (int i = 0; i < result.length; i++) {
               if (result[i] != this.value[i]) {
                  _yy_setPosition(start);
                  return _yy_fail("Value for 'chars' does not match: " + new String(result) + " != " + new String(this.value));
               }
            }
         }
         return _yy_success(result);
      }
   }

   protected class _yy_bit implements MiniParser<Boolean> {
      private final boolean hasValue;
      private final boolean value;
      public _yy_bit() {
         this.hasValue = false;
         this.value = true;
      }
      public _yy_bit(boolean _value) {
         this.hasValue = true;
         this.value = _value;
      }
      public Result<Boolean> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         while (_yy_byteIndex >= _yy_dataRead &&
                _yy_read()) {}
         if (_yy_byteIndex >= _yy_dataRead) {
            _yy_setPosition(start);
            return _yy_fail("Cannot read bytes for 'bit' primitive");
         }
         int byteindex = _yy_byteIndex % _YY_BUFFER_SIZE;
         int bufferindex = _yy_byteIndex / _YY_BUFFER_SIZE;
         byte b = _yy_data.get(bufferindex)[byteindex];
         boolean result = (((b&0xFF)>>_yy_bitIndex)&1)!=0;
         if (_yy_bitIndex == 7) _yy_byteIndex++;
         _yy_bitIndex = (_yy_bitIndex+1) & 7;
         if (this.hasValue && result != this.value) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'bit' does not match: " + result + " != " + this.value);
         }
         return _yy_success(result);
      }
   }

   protected class _yy_bits implements MiniParser<BitSet> {
      private final int length;
      private final boolean hasValue;
      private final BitSet value;
      private final _yy_bit inner = new _yy_bit();
      public _yy_bits(int _length, BitSet _value) throws IOException, ParsingException {
         if (_length < 0)
            throw new ParsingException("Negative number of bits specified: " + _length);
         this.length = _length;
         this.hasValue = (_value != null);
         this.value = _value;
      }
      public _yy_bits(int _length) throws IOException, ParsingException {
         this(_length, null);
      }
      public Result<BitSet> get() throws IOException, ParsingException {
         Position start = _yy_getPosition();
         BitSet result = new BitSet(this.length);
         for (int i = 0; i < this.length; i++) {
            Result<Boolean> bit = this.inner.get();
            if (bit.failed()) {
               _yy_setPosition(start);
               return _yy_fail("Cannot read bytes for 'bits' primitive");
            }
            result.set(i, bit.result());
         }
         if (this.hasValue && !result.equals(this.value)) {
            _yy_setPosition(start);
            return _yy_fail("Value for 'bits' does not match");
         }
         return _yy_success(result);
      }
   }

   // has no value arg!
   protected class _yy_list<T> implements MiniParser<List<T>> {
      private final int length;
      private final MiniParser<T> type;
      public _yy_list(int _length, MiniParser<T> _type) throws IOException, ParsingException {
         if (_length < 0)
            throw new ParsingException("Negative number of elements specified: " + _length);
         this.length = _length;
         this.type = _type;
      }
      public Result<List<T>> get() throws IOException, ParsingException {
         List<T> result = new ArrayList<T>(this.length);
         for (int i = 0; i < this.length; i++) {
            Position start = _yy_getPosition();
            Result<T> inner = this.type.get();
            if (inner.failed()) {
               _yy_setPosition(start);
               return _yy_fail("Cannot read all " + this.length + " list items, failed at " + i);
            }
            result.add(inner.result());
         }
         return _yy_success(result);
      }
   }

   ////// Casting methods for primitives ////////

   protected class _yy_bytes_cast_int implements MiniParser<Integer> {
      private final MiniParser<byte[]> inner;
      public _yy_bytes_cast_int(MiniParser<byte[]> _inner) {
         this.inner = _inner;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Result<byte[]> innerresult = this.inner.get();
         if (innerresult.failed()) 
            return _yy_fail(innerresult);
         byte[] bytes = innerresult.result();

         if (bytes.length > 4)
            throw new ParsingException("Too many bytes for cast to int: " + bytes.length);
         int result = 0;
         if (_yy_byteOrderBE) {
            for (int i = 0; i < bytes.length; i++) {
               result = (result<<8) | (bytes[i]&0xFF);
            }
         } else {
            for (int i = bytes.length-1; i >= 0; i--) {
               result = (result<<8) | (bytes[i]&0xFF);
            }
         }
         return _yy_success(result);
      }
   }
   
   protected class _yy_bytes_cast_short implements MiniParser<Short> {
      private final MiniParser<byte[]> inner;
      public _yy_bytes_cast_short(MiniParser<byte[]> _inner) {
         this.inner = _inner;
      }
      public Result<Short> get() throws IOException, ParsingException {
         Result<byte[]> innerresult = this.inner.get();
         if (innerresult.failed()) 
            return _yy_fail(innerresult);
         byte[] bytes = innerresult.result();

         if (bytes.length > 2)
            throw new ParsingException("Too many bytes for cast to int: " + bytes.length);
         int result = 0;
         if (_yy_byteOrderBE) {
            for (int i = 0; i < bytes.length; i++) {
               result = (result<<8) | (bytes[i]&0xFF);
            }
         } else {
            for (int i = bytes.length-1; i >= 0; i--) {
               result = (result<<8) | (bytes[i]&0xFF);
            }
         }
         return _yy_success((short)result);
      }
   }


   protected class _yy_bytes_cast_long implements MiniParser<Long> {
      private final MiniParser<byte[]> inner;
      public _yy_bytes_cast_long(MiniParser<byte[]> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<byte[]> innerresult = this.inner.get();
         if (innerresult.failed()) 
            return _yy_fail(innerresult);
         byte[] bytes = innerresult.result();

         if (bytes.length > 8)
            throw new ParsingException("Too many bytes for cast to int: " + bytes.length);
         long result = 0L;
         if (_yy_byteOrderBE) {
            for (int i = 0; i < bytes.length; i++) {
               result = (result<<8L) | (bytes[i]&0xFFL);
            }
         } else {
            for (int i = bytes.length-1; i >= 0; i--) {
               result = (result<<8L) | (bytes[i]&0xFFL);
            }
         }
         return _yy_success(result);
      }
   }

   protected class _yy_bit_cast_byte implements MiniParser<Byte> {
      private final MiniParser<Boolean> inner;
      public _yy_bit_cast_byte(MiniParser<Boolean> _inner) {
         this.inner = _inner;
      }
      public Result<Byte> get() throws IOException, ParsingException {
         Result<Boolean> innerresult = this.inner.get();
         if (innerresult.failed()) 
            return _yy_fail(innerresult);
         boolean bit = innerresult.result();
         return _yy_success((byte)(bit ? 1 : 0));
      }
   }
   protected class _yy_bit_cast_short implements MiniParser<Short> {
      private final MiniParser<Boolean> inner;
      public _yy_bit_cast_short(MiniParser<Boolean> _inner) {
         this.inner = _inner;
      }
      public Result<Short> get() throws IOException, ParsingException {
         Result<Boolean> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         boolean bit = innerresult.result();
         return _yy_success((short)(bit ? 1 : 0));
      }
   }
   protected class _yy_bit_cast_int implements MiniParser<Integer> {
      private final MiniParser<Boolean> inner;
      public _yy_bit_cast_int(MiniParser<Boolean> _inner) {
         this.inner = _inner;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Result<Boolean> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         boolean bit = innerresult.result();
         return _yy_success((bit ? 1 : 0));
      }
   }
   protected class _yy_bit_cast_long implements MiniParser<Long> {
      private final MiniParser<Boolean> inner;
      public _yy_bit_cast_long(MiniParser<Boolean> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<Boolean> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         boolean bit = innerresult.result();
         return _yy_success((bit ? 1L : 0L));
      }
   }

   private int _yy_bits_cast_int(BitSet bits) throws IOException, ParsingException {
      if (bits.length() > 32)
         throw new ParsingException("Too many bits for int: " + bits.length());
      int result = 0;
      if (this._yy_bitOrderBE) {
         for (int i = 0; i < bits.length(); i++) {
            if (bits.get(i))
               result = (result<<1) | 1;
            else
               result <<= 1;
         }
      } else {
         for (int i = bits.length()-1; i >= 0; i--) {
            if (bits.get(i))
               result = (result<<1) | 1;
            else
               result <<= 1;
         }
      }
      return result;
   }

   protected class _yy_bits_cast_byte implements MiniParser<Byte> {
      private final MiniParser<BitSet> inner;
      public _yy_bits_cast_byte(MiniParser<BitSet> _inner) {
         this.inner = _inner;
      }
      public Result<Byte> get() throws IOException, ParsingException {
         Result<BitSet> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         BitSet bits = innerresult.result();
         if (bits.length() > 8)
            throw new ParsingException("Too many bits for byte: " + bits.length());
         return _yy_success((byte)_yy_bits_cast_int(bits));
      }
   }
   protected class _yy_bits_cast_short implements MiniParser<Short> {
      private final MiniParser<BitSet> inner;
      public _yy_bits_cast_short(MiniParser<BitSet> _inner) {
         this.inner = _inner;
      }
      public Result<Short> get() throws IOException, ParsingException {
         Result<BitSet> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         BitSet bits = innerresult.result();
         if (bits.length() > 16)
            throw new ParsingException("Too many bits for short: " + bits.length());
         return _yy_success((short)_yy_bits_cast_int(bits));
      }
   }
   protected class _yy_bits_cast_int implements MiniParser<Integer> {
      private final MiniParser<BitSet> inner;
      public _yy_bits_cast_int(MiniParser<BitSet> _inner) {
         this.inner = _inner;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Result<BitSet> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         BitSet bits = innerresult.result();
         return _yy_success(_yy_bits_cast_int(bits));
      }
   }

   protected long _yy_bits_cast_long(BitSet bits) throws IOException, ParsingException {
      if (bits.length() > 64)
         throw new ParsingException("Too many bits for long: " + bits.length());
      long result = 0L;
      if (this._yy_bitOrderBE) {
         for (int i = 0; i < bits.length(); i++) {
            if (bits.get(i))
               result = (result<<1L) | 1L;
            else
               result <<= 1L;
         }
      } else {
         for (int i = bits.length()-1; i >= 0; i--) {
            if (bits.get(i))
               result = (result<<1L) | 1L;
            else
               result <<= 1L;
         }
      }
      return result;
   }

   protected class _yy_bits_cast_long implements MiniParser<Long> {
      private final MiniParser<BitSet> inner;
      public _yy_bits_cast_long(MiniParser<BitSet> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<BitSet> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         BitSet bits = innerresult.result();
         return _yy_success(_yy_bits_cast_long(bits));
      }
   }

   protected class _yy_byte_cast_short implements MiniParser<Short> {
      private final MiniParser<Byte> inner;
      public _yy_byte_cast_short(MiniParser<Byte> _inner) {
         this.inner = _inner;
      }
      public Result<Short> get() throws IOException, ParsingException {
         Result<Byte> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success((short)(innerresult.result()&0xFF));
      }
   }
   protected class _yy_byte_cast_int implements MiniParser<Integer> {
      private final MiniParser<Byte> inner;
      public _yy_byte_cast_int(MiniParser<Byte> _inner) {
         this.inner = _inner;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Result<Byte> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success(innerresult.result()&0xFF);
      }
   }
   protected class _yy_byte_cast_long implements MiniParser<Long> {
      private final MiniParser<Byte> inner;
      public _yy_byte_cast_long(MiniParser<Byte> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<Byte> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success(innerresult.result()&0xFFL);
      }
   }

   protected class _yy_chars_cast_string implements MiniParser<String> {
      private final MiniParser<char[]> inner;
      public _yy_chars_cast_string(MiniParser<char[]> _inner) {
         this.inner = _inner;
      }
      public Result<String> get() throws IOException, ParsingException {
         Result<char[]> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         char[] chars = innerresult.result();
         return _yy_success(new String(chars));
      }
   }
   protected class _yy_chars_cast_stringbuilder implements MiniParser<StringBuilder> {
      private final MiniParser<char[]> inner;
      public _yy_chars_cast_stringbuilder(MiniParser<char[]> _inner) {
         this.inner = _inner;
      }
      public Result<StringBuilder> get() throws IOException, ParsingException {
         Result<char[]> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         char[] chars = innerresult.result();
         return _yy_success(new StringBuilder(new String(chars)));
      }
   }
   protected class _yy_chars_cast_stringbuffer implements MiniParser<StringBuffer> {
      private final MiniParser<char[]> inner;
      public _yy_chars_cast_stringbuffer(MiniParser<char[]> _inner) {
         this.inner = _inner;
      }
      public Result<StringBuffer> get() throws IOException, ParsingException {
         Result<char[]> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         char[] chars = innerresult.result();
         return _yy_success(new StringBuffer(new String(chars)));
      }
   }

   protected class _yy_float_cast_double implements MiniParser<Double> {
      private final MiniParser<Float> inner;
      public _yy_float_cast_double(MiniParser<Float> _inner) {
         this.inner = _inner;
      }
      public Result<Double> get() throws IOException, ParsingException {
         Result<Float> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success((double)innerresult.result());
      }
   }

   protected class _yy_int_cast_long implements MiniParser<Long> {
      private final MiniParser<Integer> inner;
      public _yy_int_cast_long(MiniParser<Integer> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<Integer> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success(innerresult.result()&0xFFFFFFFFL);
      }
   }

   protected class _yy_short_cast_int implements MiniParser<Integer> {
      private final MiniParser<Short> inner;
      public _yy_short_cast_int(MiniParser<Short> _inner) {
         this.inner = _inner;
      }
      public Result<Integer> get() throws IOException, ParsingException {
         Result<Short> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success(innerresult.result()&0xFFFF);
      }
   }
   protected class _yy_short_cast_long implements MiniParser<Long> {
      private final MiniParser<Short> inner;
      public _yy_short_cast_long(MiniParser<Short> _inner) {
         this.inner = _inner;
      }
      public Result<Long> get() throws IOException, ParsingException {
         Result<Short> innerresult = this.inner.get();
         if (innerresult.failed()) return _yy_fail(innerresult);
         return _yy_success(innerresult.result()&0xFFFFL);
      }
   }

   /////// Value checkers
   protected boolean _yy_checkvalue(byte lhs, byte rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(short lhs, short rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(int lhs, int rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(char lhs, char rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(float lhs, float rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(double lhs, double rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(long lhs, long rhs) {
      return lhs == rhs;
   }
   protected boolean _yy_checkvalue(boolean lhs, boolean rhs) {
      return lhs == rhs;
   }

   protected boolean _yy_checkvalue(Object lhs, Object rhs) {
      // check for nullness
      if (lhs==null && rhs==null)
         return true;
      else if (lhs==null || rhs==null)
         return false;

      boolean lhsIsArray = lhs.getClass().isArray();
      boolean rhsIsArray = lhs.getClass().isArray();
      if (lhsIsArray && rhsIsArray) {
         // both arrays, check types
         if ((lhs instanceof byte[]) && (rhs instanceof byte[])) {
            return Arrays.equals((byte[])lhs, (byte[])rhs);
         }
         else if ((lhs instanceof char[]) && (rhs instanceof char[])) {
            return Arrays.equals((char[])lhs, (char[])rhs);
         }
         else if ((lhs instanceof double[]) && (rhs instanceof double[])) {
            return Arrays.equals((double[])lhs, (double[])rhs);
         }
         else if ((lhs instanceof float[]) && (rhs instanceof float[])) {
            return Arrays.equals((float[])lhs, (float[])rhs);
         }
         else if ((lhs instanceof int[]) && (rhs instanceof int[])) {
            return Arrays.equals((int[])lhs, (int[])rhs);
         }
         else if ((lhs instanceof long[]) && (rhs instanceof long[])) {
            return Arrays.equals((long[])lhs, (long[])rhs);
         }
         else if ((lhs instanceof short[]) && (rhs instanceof short[])) {
            return Arrays.equals((short[])lhs, (short[])rhs);
         }
         else if ((lhs instanceof boolean[]) && (rhs instanceof boolean[])) {
            return Arrays.equals((boolean[])lhs, (boolean[])rhs);
         }
         else {
            // object arrays
            Object[] lhsArray = (Object[])lhs;
            Object[] rhsArray = (Object[])rhs;
            if (lhsArray.length != rhsArray.length)
               return false;
            for (int i = 0; i < lhsArray.length; i++) {
               if (!_yy_checkvalue(lhsArray[i], rhsArray[i]))
                  return false;
            }
            return true;
         }
      } else if (lhsIsArray || rhsIsArray) {
         // only one is array, must be false
         return false;
      } else {
         // both not arrays, call .equals
         return lhs.equals(rhs);
      }
   }

   /////

   protected boolean _yy_atEOF() throws IOException, ParsingException {
      int byteIndex = this._yy_byteIndex;
      if (this._yy_bitIndex != 0)
         byteIndex++;
      while (byteIndex >= this._yy_dataRead &&
             this._yy_read()) {}
      return (byteIndex >= this._yy_dataRead);
   }
}
