package pigbarf;

public class Buffer {
	private int index;
	private final CharSequence data;
	
	public Buffer(CharSequence _data) {
		this.data = _data;
		this.index = 0;
	}
	
	public void skipWS() {
		while (!atEOF()) {
			char c = this.data.charAt(this.index);
			switch (c) {
			case ' ':
			case '\t':
			case '\n':
			case '\r':
				this.index++;
				break;
			default:
				return;
			}
		}
	}
	
	public boolean next(String str) {
		if (!hasN(str.length()))
			return false;
		for (int i = 0; i < str.length(); i++) {
			if (this.data.charAt(this.index+i) != str.charAt(i))
				return false;
		}
		return true;
	}

	public int getIndex() {return this.index;}
	public void setIndex(int i) {
		if (i < 0 || i > this.data.length()) 
			throw new IndexOutOfBoundsException();
		this.index = i;
	}
	
	public void assertN(int n) throws MetaParsingException {
		if (this.index+n > this.data.length())
			throw new MetaParsingException("Premature EOF");
	}
	public boolean hasN(int n) {
		return this.index+n <= this.data.length();
	}
	public char peek() throws MetaParsingException {
		assertN(1);
		return this.data.charAt(this.index);
	}
	public char get() throws MetaParsingException {
		assertN(1);
		return this.data.charAt(this.index++);
	}
	public CharSequence peekN(int n) throws MetaParsingException {
		assertN(n);
		return this.data.subSequence(this.index, this.index+n);
	}
	public CharSequence getN(int n) throws MetaParsingException {
		assertN(n);
		CharSequence result = this.data.subSequence(this.index, this.index+n);
		this.index += n;
		return result;
	}
	public boolean atEOF() {return this.index == this.data.length();}
}
