package pigbarf;

import java.util.ArrayList;
import java.util.List;

public class FunctionType extends Type {
	private final SimpleType returnType;
	private final List<SimpleType> paramTypes = new ArrayList<SimpleType>();
	
	public FunctionType(SimpleType _return, List<SimpleType> _params) {
		this.returnType = _return;
		this.paramTypes.addAll(_params);
	}
	
	public boolean isFunction() {return true;}
	public FunctionType getFunctionSelf() {return this;}
	
	public int getNumParameters() {return this.paramTypes.size();}
	public SimpleType getParameterType(int i) {return this.paramTypes.get(i);}
	public SimpleType getReturnType() {return this.returnType;}

	public boolean equals(Object o) {
		if (!(o instanceof Type))
			return false;
		Type t = (Type)o;
		if (!t.isFunction())
			return false;
		FunctionType f = t.getFunctionSelf();
		if (!f.getReturnType().equals(this.getReturnType()))
			return false;
		if (f.getNumParameters() != this.getNumParameters())
			return false;
		for (int i = 0; i < this.getNumParameters(); i++) {
			if (!f.getParameterType(i).equals(this.getParameterType(i)))
				return false;
		}
		return true;
	}
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append('(');
		for (int i = 0; i < this.paramTypes.size(); i++) {
			if (i>0) builder.append(',');
			builder.append(this.paramTypes.get(i));
		}
		return builder.append(") ").append(this.returnType).toString();
	}
	public int hashCode() {
		return this.paramTypes.hashCode()*3 + this.returnType.hashCode()*7;
	}
}
