package pigbarf;

import java.util.HashMap;
import java.util.Map;

public class ListRuleItem extends ValueRuleItem {
	private final ValueRuleItem inner;
	private final String length;
	
	public ListRuleItem(ValueRuleItem _inner, String _length) {
		this.inner = _inner;
		this.length = _length;
	}

	public ValueRuleItem getInnerItem() {return this.inner;}
	public String getLengthValue() {return this.length;}

	public boolean isList() {return true;}
	public ListRuleItem getListSelf() {return this;}
	
	private static Map<String,String> wrapperMapper = new HashMap<String,String>();
	static {
		wrapperMapper.put("boolean", "java.lang.Boolean");
		wrapperMapper.put("byte", "java.lang.Byte");
		wrapperMapper.put("char", "java.lang.Character");
		wrapperMapper.put("double", "java.lang.Double");
		wrapperMapper.put("float", "java.lang.Float");
		wrapperMapper.put("int", "java.lang.Integer");
		wrapperMapper.put("long", "java.lang.Long");
		wrapperMapper.put("short", "java.lang.Short");
	}

   public static String wrapType(String type) {
      if (wrapperMapper.containsKey(type))
         return wrapperMapper.get(type);
      else if (type.equals("void"))
         return "java.lang.Void";
      else
         return type;
   }

	public SimpleType getType() {
		SimpleType innerType = this.inner.getType();
		String typeParam;
		if (wrapperMapper.containsKey(innerType.getType()))
			typeParam = wrapperMapper.get(innerType.getType());
		else
			typeParam = innerType.getType();
		return new SimpleType("java.util.List<" + typeParam + ">");
	}

   public String toString() {
      return "list(type=" + this.inner + ", length=" + this.length + ")";
   }
}
