package pigbarf;

import java.util.ArrayList;
import java.util.List;

public class NonterminalRuleItem extends ValueRuleItem {
	private final String name;
	private final List<String> actuals = new ArrayList<String>();
	private Rule rule; // starts as null, then gets assigned
	// actuals are strings of uninterpreted java code, possibly including $# values
	
	public NonterminalRuleItem(String _name, List<String> _actuals) {
		this.name = _name;
		this.actuals.addAll(_actuals);
	}
	
	public void setRule(Rule _rule) throws MetaParsingException {
		if (this.rule != null)
			throw new MetaParsingException("Already have a rule assigned");
		if (!_rule.getName().equals(this.name))
			throw new MetaParsingException("Names don't match");
		if (this.actuals.size() != _rule.getNumParameters())
			throw new MetaParsingException("Wrong number of parameters for nonterminal " + this.name);
		this.rule = _rule;
	}
	
	public SimpleType getType() {
		if (this.rule == null)
			throw new UnsupportedOperationException();
		Type type = this.rule.getType();
		if (type.isFunction())
			return type.getFunctionSelf().getReturnType();
		else
			return type.getSimpleSelf();
	}
	
	public boolean isNonterminal() {return true;}
	public NonterminalRuleItem getNonterminalSelf() {return this;}

	public String getName() {return this.name;}
	public int getNumActuals() {return this.actuals.size();}
	public String getActual(int i) {return this.actuals.get(i);}

   public String toString() {
      StringBuilder builder = new StringBuilder(this.name);
      if (this.actuals.size() > 0) {
         builder.append('(');
         for (int i = 0; i < this.actuals.size(); i++) {
            if (i>0) builder.append(", ");
            builder.append(this.actuals.get(i));
         }
         builder.append(')');
      }
      return builder.toString();
   }
}
