package pigbarf;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class PrimitiveRuleItem extends ValueRuleItem {
	private final Primitive primitive;
	private final Map<String,String> args;
	
	public PrimitiveRuleItem(Primitive _primitive, Map<String,String> _args) 
	throws MetaParsingException {
		PrimitiveAmbassador.checkArgs(_primitive, _args);
		this.primitive = _primitive;
		this.args = Collections.unmodifiableMap(new HashMap<String,String>(_args));
	}
	
	public boolean isPrimitive() {return true;}
	public PrimitiveRuleItem getPrimitiveSelf() {return this;}
	
	public Primitive getPrimitive() {return this.primitive;}

	public Set<String> getArgumentNames() {return this.args.keySet();}
	public String getArgumentValue(String argName) {return this.args.get(argName);}
	public boolean hasArgument(String argName) {return this.args.containsKey(argName);}
	
	public SimpleType getType() {
		if (this.args.containsKey("cast")) {
         String cast = this.args.get("cast");
         if (cast.equals("String"))
            cast = "java.lang.String";
         else if (cast.equals("StringBuilder"))
            cast = "java.lang.StringBuilder";
         else if (cast.equals("StringBuffer"))
            cast = "java.lang.StringBuffer";
			return new SimpleType(cast);
      } else
			return this.primitive.getDefaultType();
	}

   public String toString() {
      StringBuilder builder = new StringBuilder(this.primitive.name().toLowerCase());
      if (this.args.size() > 0) {
         builder.append('(');
         boolean gotone = false;
         for (String key : this.args.keySet()) {
            if (gotone) builder.append(", ");
            gotone = true;
            builder.append(key).append('=').append(this.args.get(key));
         }
         builder.append(')');
      }
      return builder.toString();
   }
}
