package pigbarf;

import java.util.*;

public class SwitchRuleItem extends ValueRuleItem {
   public static class Case {
      public final String match;
      public final ValueRuleItem rule;
      public Case(String _match, ValueRuleItem _rule) {
         this.match = _match;
         this.rule = _rule;
      }
   }

   private final String target;
   private final List<Case> cases = new ArrayList<Case>();
   private final ValueRuleItem defaultCase;
   
   public SwitchRuleItem(String _target, Collection<Case> _cases, ValueRuleItem _defaultCase) {
      this.target = _target;
      this.cases.addAll(_cases);
      this.defaultCase = _defaultCase;
   }

   public String getTargetString() {return this.target;}
   public int getNumCases() {return this.cases.size();}
   public Case getCase(int index) {return this.cases.get(index);}
   public ValueRuleItem getDefaultCase() {return this.defaultCase;}

   public boolean isSwitch() {return true;}
   public SwitchRuleItem getSwitchSelf() {return this;}

   public SimpleType getType() {
      if (this.cases.size() > 0) {
         return this.cases.get(0).rule.getType();
      } else if (this.defaultCase != null) {
         return this.defaultCase.getType();
      } else {
         throw new UnsupportedOperationException();
      }
   }

   public String toString() {
      StringBuilder builder = new StringBuilder("switch [[").append(this.target).append("]] {");
      for (Case c : this.cases) {
         builder.append(" [[").append(c.match).append("]] -> ").append(c.rule).append(" ;;");
      }
      if (this.defaultCase != null)
         builder.append(" default -> ").append(this.defaultCase).append(" ;;");
      return builder.append(" }").toString();
   }
}
